<?php

// IPplan v6.00-BETA2
// Aug 24, 2001
//
// This program is free software; you can redistribute it and/or modify
// it under the terms of the GNU General Public License as published by
// the Free Software Foundation; either version 2 of the License, or
// (at your option) any later version.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// GNU General Public License for more details.
//
// You should have received a copy of the GNU General Public License
// along with this program; if not, write to the Free Software
// Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA 02111-1307, USA.
//

function next_subnets ($ip, $size, $num) {

	$subnets = array();
	array_push($subnets,$ip);
	if ( Net_IPv4::validateIP($ip) ) {
		$size = pow(2, 32 - $size);
		$ip = inet_aton($ip);
		for ($i=2; $i <= $num; $i++) {
			$newbase = inet_ntoa( $ip+(($i-1)*$size) );
			if ( !preg_match("/^256\.*/",$newbase) )
				array_push($subnets, $newbase);
		}
	} else {
	  $down = ceil( (log($num)/log(2)) );
	  $parts = explode(":",uncompress($ip)); $bits = "";
	  for ($i=0; $i < 8; $i++) {
		$parts[$i] = base_convert($parts[$i],16,2);
		$parts[$i] = str_pad($parts[$i],16,"0",STR_PAD_LEFT);
		$bits .= $parts[$i];
	  }
	  $change = substr($bits,$size-$down,$down);
	  $new = $change;
	  for ($i=1; $i < $num; $i++) {
		$new = decbin(bindec($new)+1);
		$new = str_pad($new,$down,"0",STR_PAD_LEFT);
		$bits = substr_replace($bits, $new, $size-$down, $down);
		$tempnet = array(); $start = 0; $step = 16;
		while ( count($tempnet) < 8 ) {
		  array_push($tempnet, base_convert(substr($bits,$start,$step),2,16) );
		  $start += $step;
		}
		array_push($subnets, implode(":",$tempnet));
	  }
	}
	return $subnets;
}

require_once("../ipplanlib.php");
require_once("/usr/share/php/modules/adodb/adodb.inc.php");
require_once("../class.dbflib.php");
require_once("../layout/class.layout");
require_once("../auth.php");
require_once("../class.templib.php");
require_once("../IPv4.php");
require_once("../IPv6.php");

$auth = new SQLAuthenticator(REALM, REALMERROR);

//IPv4 and IPv6 classes
$ipv4 = new Net_IPv4;
$ipv6 = new Net_IPv6;
$iptype = 0;

// And now perform the authentication
$grps=$auth->authenticate();

// save the last customer used
// must set path else Netscape gets confused!
setcookie("ipplanCustomer","$cust",time() + 10000000, "/");
setcookie("ipplanGroup","$admingrp",time() + 10000000, "/");

// set language
isset($_COOKIE["ipplanLanguage"]) && myLanguage($_COOKIE['ipplanLanguage']);

//setdefault("window",array("bgcolor"=>"white"));
//setdefault("table",array("cellpadding"=>"0"));
//setdefault("text",array("size"=>"2"));

$title=my_("Create subnet results");
newhtml($p);
$w=myheading($p, $title, true);

// explicitly cast variables as security measure against SQL injection
list($cust, $admingrp, $ipaddr, $num, $descrip, $size, $addhostinfo, $addnmapinfo, $dhcp, $findfree) = myRegister("I:cust S:admingrp S:ipaddr I:num S:descrip I:size I:addhostinfo I:addnmapinfo I:dhcp S:findfree");
list($userfld) = myRegister("A:userfld");  // for template fields

$descrip=trim($descrip);

// must only check once - might need to create multiple nets
$nodescrip=0;
if (strlen($descrip) == 0) {
   $nodescrip=1;
}
if ( ( $iptype == 1 and ($num < 1 or $num > 255) ) ) {
   myError($w,$p, my_("Number of subnets to create is out of bounds."));
}

// basic sequence is connect, search, interpret search
// result, close connection
$ds=new IPplanDbf() or myError($w,$p, my_("Could not connect to database"));
// error checks
if (!$ipaddr)
   myError($w,$p, my_("IP address may not be blank"));
else if ( testIP($ipaddr) )
   myError($w,$p, my_("Invalid IP address!"));
else if (!$size)
   myError($w,$p, my_("Size may not be zero"));
else if ($size > 1) {
	if ( TestBaseAddr($ipaddr,$size) )
		myError($w,$p, my_("Invalid base address!"));
}

//checking whether we are dealing with IPv4 or IPv6
$iptype = $ipv4->validateIP($ipaddr) ? AFI_INET : AFI_INET6;
if ( $iptype == AFI_INET6 ) {
  $ipaddr = uncompress($ipaddr);
}

if ($dhcp) {
    // use dhcp template (for additional subnet information)
    $template=new IPplanIPTemplate("basetemplate-dhcp", $cust);
}
else {
    // use base template (for additional subnet information)
    $template=new IPplanIPTemplate("basetemplate", $cust);
}

$info="";
if ($template->is_error() == FALSE) {
    // PROBLEM HERE: if template create suddenly returns error (template file
    // permissions, xml error etc), then each submit thereafter will erase
    // previous contents - this is not good
    $template->Merge($userfld);
    $err=$template->Verify($w);

    if ($template->is_blank() == FALSE) {
        $info=$template->encode();
    }
}

if ($err) {
    printhtml($p);
    exit;
}

if ($addhostinfo) {
   // increase time limit for scans - will have no effect in safe mode is on
   @set_time_limit(90);
}

$warn=0;

//getting all my bases
$base = next_subnets($ipaddr,$size,$num);
$cnt_base = count($base);
for ($i=0; $i < $cnt_base; $i++) {
    // create description if none
    if ($nodescrip) {
        $descrip="NET-".str_replace( ".", "-", $base[$i] );
    }
    // test if subnet to create is within bounds
    foreach ($grps as $value) {
        if ($extst = $ds->TestBounds($base[$i], $size, $value)) {
            // got an overlap, allowed to create
            break;
        }
    }
    // could not find new subnet within any of the defined bounds
    // so do not create
    if (!$extst) {
        $warn=1;
        myError($w,$p, sprintf(my_("Subnet %s not created - out of defined authority boundary"), $base[$i])."\n", FALSE);
        continue;
    }

    $restemp = $ds->GetDuplicateSubnet($base[$i], $size, $cust);
    if ( count($restemp) > 0 ) {
        $warn=1;
        myError($w,$p, sprintf(my_("Subnet %s could not be created - possibly overlaps with an existing subnet"),$base[$i])."\n", FALSE);
    } else {

        // find subnets from other customers that overlap
        $result = $ds->GetDuplicateSubnetAll($base[$i], $size, $grps);
        if ( count($result) > 0 ) {
            $warn=1;
            insert($w,textb(my_("WARNING: ")));
            insert($w,text(sprintf(my_("Subnet %s overlaps with the following subnets from other customers.  This may not be a problem as the subnet may occur in both customers routing tables:"),$base[$i])));
            insert($w,block("<p>"));

            // create a table
            insert($w,$t = table(array("cols"=>"3",
                            "class"=>"outputtable")));
            // draw heading
            setdefault("cell",array("class"=>"heading"));
            insert($t,$c = cell());
            insert($c,text(my_("Base address")));
            insert($t,$c = cell());
            insert($c,text(my_("Subnet description")));
            insert($t,$c = cell());
            insert($c,text(my_("Customer")));

            $cnt_result = count($result);
            for ($j=0; $j < $cnt_result; $j++) {
                setdefault("cell",array("class"=>color_flip_flop()));

                insert($t,$c = cell());
                insert($c,text($result[$j]["baseaddr"]));

                insert($t,$c = cell());
                insert($c,text($result[$j]["descrip"]));

                insert($t,$c = cell());
                insert($c,text($result[$j]["custdescrip"]));
            }

            setdefault("cell", "");
            insert($w,block("<p>"));
        }

        // check if user belongs to customer admin group
        $result=$ds->GetCustomerGrp($cust);
        // can only be one row - does not matter if nothing is 
        // found as array search will return false
        $row = $result->FetchRow();
        if (!in_array($row["admingrp"], $grps)) {
            myError($w,$p, my_("You may not create a subnet for this customer as you are not a member of the customers admin group"));
        }

        $ds->DbfTransactionStart();
        // use the first group user belongs to create subnet
        if ($id = $ds->CreateSubnet($base[$i], $size, $descrip, $cust, $dhcp, $admingrp)) {
            $ds->AuditLog(array("event"=>170, "action"=>"create subnet", 
                    "descrip"=>$descrip, "user"=>getAuthUsername(), "baseaddr"=>$base[$i],
                    "size"=>$size, "cust"=>$cust));

            insert($w,text(sprintf(my_("Subnet %s created"), $base[$i])));

            // fill new subnet with nmap
            if (NMAP != "" and $addnmapinfo and $size >= 22 and $size <= 30) {
                if (ProcessNmap($ds, $base[$i], $id, $size)) {
                    insert($w,textbr());
                    myError($w,$p, my_("NMAP data not in XML format or XML not available"), FALSE);
                }
            }
            // fill new subnet with DNS info
            else if ($addhostinfo and $size > 0) {
			  $ips = next_ips($base[$i],$size);
			  $cnt_ips = count($ips);
              // remember to skip broadcast and network addresses!
              for ($j=0; $j < $cnt_ips; $j++) {
				$hname = "";
				if ( $iptype == 1 ) {
				  $hname = gethostbyaddr($ips[$j]);
                  if ( strcmp($hname,$ips[$j]) != 0) {
					$ds->AddIP($ips[$j], $id, "", "", "", "", "", "", $hname, "");
				  }
				} else {
				  $ip = $ipv6->getNetmask($base[$i], $size);
				  $hname = gethostbyaddr( $ip );
                  if ($hname != $ip)
					$ds->AddIP($ip, $id, "", "", "", "", "", "", $hname, "");
				}
              }
            }

            if (!empty($info)) {
                $result = &$ds->ds->Execute("INSERT INTO baseadd
                        (info, baseindex)
                        VALUES
                        (".$ds->ds->QMagic($info).", $id)");
            }

            $ds->DbfTransactionEnd();

            insert($w,textbr());
        }
        else {
            myError($w,$p, sprintf(my_("Subnet %s could not be created"), $base[$i]));
        }

    }
} // end of for loop for multiple subnet creation

// add link if script was called from findfree
// only if all subnets created without error or warning
if (!$warn and $findfree) {
    // $findfree contains HTTP_REFERER which is a full URI already - OK
    header("Location: ".base64_decode($findfree));
    exit;
}
if (!$warn and $num==1) {
    header("Location: ".location_uri("displaybase.php?cust=$cust&ipaddr=$ipaddr"));
    exit;
}

printhtml($p);

// $base=baseaddr of subnet, $id=baseindex
function ProcessNmap($ds, $base, $id, $size) {

   global $addhostinfo;

   $resarr=array();

   if ($addhostinfo)
      $command=NMAP." -sP ".escapeshellarg( $base."/".$size )." -oX -";
   else
      $command=NMAP." -n -sP ".escapeshellarg( $base."/".$size )." -oX -";

   exec($command, $resarr, $retval);
   // did NMAP fail due to safe mode or other error?
   if ($retval) { 
       return 1;
   }
   else { // no error
       require_once("../xmllib.php");

       $input = implode("",$resarr);
       // nmap parser always returns arrays for tags of form
       // [tagname][0...x][element]
       // array index will mostly be zero if one as most results
       // return 1 tag
       $xml_parser = new xmlnmap("HOST");
       if (!$xml_parser->parser) {
           return 1;  // XML parser failure - probably not compiled in
       }
       $output=$xml_parser->parse($input);
       if (!$output) {
           return 1;  // not XML format
       }

       foreach ($output as $value) {
           if ($value["STATUS"][0]["STATE"]=="up") {
               // need to loop through ADDR array here! Check that 
               // ["ADDRESS"][0]["ADDRTYPE"]=="ipv4" or 
               // ["ADDRESS"][0]["ADDRTYPE"]=="mac"
               $newbase = $value["ADDRESS"][0]["ADDR"];
               $newmac="";
               if ($value["ADDRESS"][1]["ADDRTYPE"]=="mac") {
                   $newmac=str_replace(array(":", "-", " "), "", $value["ADDRESS"][1]["ADDR"]);
               }
               if(empty($value["OSMATCH"][0]["NAME"])) {
                   $newuser="";
               }
               else {
                   $newuser=$value["OSMATCH"][0]["NAME"];
               }
               $newdescrip="active";
               if(!empty($value["HOSTNAME"][0]["NAME"])) {
                   $newhname=$value["HOSTNAME"][0]["NAME"];
               }
               else {
                   $newhname="";
               }

               // check within range of subnet before adding to ignore
               // broadcast and network addresses
               if ( (Net_IPv4::validateIP($newbase) && Net_IPv4::ipInNetwork($newbase,$base."/".$size)) || 
		    (Net_IPv6::checkIPv6($newbase) && Net_IPv6::isInNetmask($newbase,$base,$size)) ) {
                   $ds->AddIP($newbase, $id, $newuser, "", "", $newmac, $newdescrip, "", $newhname, "");
                   // address was polled? So add polled status
                   $ds->UpdateIPPoll($id, $newbase);
               }
           }
       }

       return 0;
   }

}
?>
